package de.ozgcloud.apilib.vorgang.dummy;

import java.time.ZonedDateTime;
import java.time.temporal.ChronoUnit;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import de.ozgcloud.apilib.common.errorhandling.NotFoundException;
import de.ozgcloud.apilib.file.dummy.DummyOzgCloudFileService;
import de.ozgcloud.apilib.vorgang.OzgCloudAntragsteller;
import de.ozgcloud.apilib.vorgang.OzgCloudEingang;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgang;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangHeader;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangId;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangQuery;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangService;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangStatus;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangStub;
import de.ozgcloud.apilib.vorgang.Page;

public class DummyVorgangService implements OzgCloudVorgangService {

	private final Map<OzgCloudVorgangId, OzgCloudVorgang> idToVorgang;
	private final List<OzgCloudVorgangStub> vorgangs;

	private static final String VORGANG_NAME_1 = "Antrag auf einen Waffelschein";
	private static final String VORGANG_NR_1 = "1234";
	private static final OzgCloudVorgangHeader HEADER_1 = OzgCloudVorgangHeader.builder()
			.status(OzgCloudVorgangStatus.from("NEU"))
			.createdAt(ZonedDateTime.now())
			.aktenzeichen("ABC-123-04")
			.build();
	private static final OzgCloudEingang EINGANG_1 = OzgCloudEingang.builder()
			.antragsteller(OzgCloudAntragsteller.builder().build())
			.representations(List.of(DummyOzgCloudFileService.DUMMY_FILE))
			.build();

	private static final String VORGANG_NAME_2 = "Antrag auf Führung eines Kampfhamsters";
	private static final String VORGANG_NR_2 = "5678";
	private static final OzgCloudVorgangHeader HEADER_2 = OzgCloudVorgangHeader.builder()
			.status(OzgCloudVorgangStatus.from("IN_BEARBEITUNG"))
			.createdAt(ZonedDateTime.now().minus(1, ChronoUnit.DAYS))
			.aktenzeichen("ABC-546-12")
			.build();
	private static final OzgCloudEingang EINGANG_2 = OzgCloudEingang.builder().build();

	public DummyVorgangService() {
		idToVorgang = buildMap();
		vorgangs = initList();
	}

	private Map<OzgCloudVorgangId, OzgCloudVorgang> buildMap() {
		return Map.of(
				OzgCloudVorgangId.from("1"), OzgCloudVorgang.builder()
						.id(OzgCloudVorgangId.from("1"))
						.vorgangName(VORGANG_NAME_1)
						.vorgangNummer(VORGANG_NR_1)
						.header(HEADER_1)
						.eingang(EINGANG_1)
						.build(),
				OzgCloudVorgangId.from("2"), OzgCloudVorgang.builder()
						.id(OzgCloudVorgangId.from("2"))
						.vorgangName(VORGANG_NAME_2)
						.vorgangNummer(VORGANG_NR_2)
						.header(HEADER_2)
						.eingang(EINGANG_2)
						.build());
	}

	private List<OzgCloudVorgangStub> initList() {
		return List.of(
				OzgCloudVorgangStub.builder().id(OzgCloudVorgangId.from("1"))
						.vorgangName(VORGANG_NAME_1)
						.vorgangNummer(VORGANG_NR_1)
						.header(HEADER_1)
						.build(),
				OzgCloudVorgangStub.builder().id(OzgCloudVorgangId.from("2"))
						.vorgangName(VORGANG_NAME_2)
						.vorgangNummer(VORGANG_NR_2)
						.header(HEADER_2)
						.build());
	}

	@Override
	public OzgCloudVorgang getById(OzgCloudVorgangId vorgangId) {
		var vorgang = idToVorgang.get(vorgangId);

		if (Objects.isNull(vorgang)) {
			throw new NotFoundException(vorgangId, "Vorgang");
		}

		return vorgang;
	}

	@Override
	public List<OzgCloudVorgangStub> search(String searchString) {
		return vorgangs;
	}

	@Override
	public List<OzgCloudVorgangStub> search(String searchString, Page page) {
		return vorgangs;
	}

	@Override
	public List<OzgCloudVorgangStub> find(OzgCloudVorgangQuery query, Page page) {
		return vorgangs;
	}
}
