/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.apilib.vorgang.grpc;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.Spy;

import de.ozgcloud.apilib.vorgang.OzgCloudServiceKontoTestFactory;

class OzgCloudServiceKontoMapperTest {

	@Spy
	private OzgCloudServiceKontoMapper mapper = Mappers.getMapper(OzgCloudServiceKontoMapper.class);

	@Nested
	class TestGetServiceKonto {

		@Test
		void shouldCallToServiceKonto() {
			var serviceKonto = OzgCloudServiceKontoTestFactory.create();
			doReturn(serviceKonto).when(mapper).fromServiceKonto(any());
			var grpcServiceKonto = GrpcServiceKontoTestFactory.create();

			mapper.getServiceKonto(GrpcVorgangHeadTestFactory.createBuilder().setServiceKonto(grpcServiceKonto).build());

			verify(mapper).fromServiceKonto(grpcServiceKonto);
		}

		@Test
		void shouldReturnNull() {
			var result = mapper.getServiceKonto(GrpcVorgangHeadTestFactory.createBuilder().clearServiceKonto().build());

			assertThat(result).isNull();
		}
	}

	@Nested
	class TestFromServiceKonto {

		@Test
		void shouldMapServiceKonto() {
			var result = mapper.fromServiceKonto(GrpcServiceKontoTestFactory.create());

			assertThat(result).usingRecursiveComparison().isEqualTo(OzgCloudServiceKontoTestFactory.create());
		}
	}

}