package de.ozgcloud.client.autoconfigure;

import java.util.Collection;
import java.util.Map;

import org.mapstruct.factory.Mappers;
import org.springframework.boot.autoconfigure.AutoConfiguration;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;

import de.ozgcloud.apilib.alfa.AlfaService;
import de.ozgcloud.apilib.alfa.CommandAlfaService;
import de.ozgcloud.apilib.alfa.dummy.DummyAlfaService;
import de.ozgcloud.apilib.client_attribute.OzgCloudClientAttributeService;
import de.ozgcloud.apilib.client_attribute.grpc.OzgCloudClientAttributeGrpcService;
import de.ozgcloud.apilib.client_attribute.grpc.OzgCloudClientAttributeMapper;
import de.ozgcloud.apilib.common.callcontext.DefaultOzgCloudCallContextProvider;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextProvider;
import de.ozgcloud.apilib.common.command.OzgCloudCommandService;
import de.ozgcloud.apilib.common.command.grpc.CommandMapper;
import de.ozgcloud.apilib.common.command.grpc.GrpcOzgCloudCommandService;
import de.ozgcloud.apilib.file.OzgCloudFileService;
import de.ozgcloud.apilib.file.grpc.GrpcOzgCloudFileService;
import de.ozgcloud.apilib.file.grpc.OzgCloudFileMapper;
import de.ozgcloud.apilib.user.GrpcOzgCloudUserProfileService;
import de.ozgcloud.apilib.user.OzgCloudUserProfileService;
import de.ozgcloud.apilib.user.UserProfileMapper;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangIdMapper;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangService;
import de.ozgcloud.apilib.vorgang.dummy.DummyVorgangService;
import de.ozgcloud.apilib.vorgang.grpc.GrpcOzgCloudVorgangService;
import de.ozgcloud.apilib.vorgang.grpc.OzgCloudVorgangMapper;
import de.ozgcloud.apilib.vorgang.grpc.OzgCloudVorgangQueryMapper;
import de.ozgcloud.apilib.vorgang.grpc.OzgCloudVorgangStubMapper;
import de.ozgcloud.user.grpc.userprofile.UserProfileServiceGrpc;
import de.ozgcloud.vorgang.grpc.binaryFile.BinaryFileServiceGrpc;
import de.ozgcloud.vorgang.grpc.clientAttribute.ClientAttributeServiceGrpc;
import de.ozgcloud.vorgang.grpc.command.CommandServiceGrpc;
import de.ozgcloud.vorgang.vorgang.VorgangServiceGrpc;
import io.grpc.ManagedChannelBuilder;
import lombok.RequiredArgsConstructor;
import net.devh.boot.grpc.client.autoconfigure.GrpcClientAutoConfiguration;
import net.devh.boot.grpc.client.config.GrpcChannelProperties;
import net.devh.boot.grpc.client.config.GrpcChannelsProperties;

@AutoConfiguration(before = GrpcClientAutoConfiguration.class)
@ComponentScan("de.ozgcloud.client.autoconfigure")
@RequiredArgsConstructor
public class OzgCloudClientAutoConfiguration {

	private static final String CLIENT_NAME_VORGANG_MANAGER = "vorgang-manager";
	private static final String CLIENT_NAME_FILE_MANAGER = "file-manager";
	private static final String CLIENT_NAME_COMMAND_MANAGER = "ozgcloud-command-manager";
	private static final String CLIENT_NAME_USER_MANAGER = "ozgcloud-user-manager";

	private final OzgCloudVorgangManagerProperties vorgangManagerProperties;
	private final OzgCloudFileManagerProperties fileManagerProperties;
	private final OzgCloudCommandManagerProperties commandManagerProperties;
	private final OzgCloudUserManagerProperties userManagerProperties;

	// @Bean
	// @ConditionalOnProperty("ozgcloud.vorgang-manager.address")
	GrpcChannelsProperties channelProperties(GrpcChannelsProperties properties) {
		// var properties = new GrpcChannelsProperties();
		var clientMap = properties.getClient();

		addVorgangManager(clientMap);
		addFileManager(clientMap);
		addCommandManager(clientMap);

		return properties;
	}

	@Bean
	@ConditionalOnProperty("ozgcloud.command-manager.address")
	GrpcChannelConfigurator commandConfigurator() {
		return GrpcChannelConfigurator.builder()
				.clientName(CLIENT_NAME_COMMAND_MANAGER)
				.address(commandManagerProperties.getAddress())
				.negotiationType(commandManagerProperties.getNegotiationType())
				.build();
	}

	@Bean
	@ConditionalOnProperty("ozgcloud.user-manager.address")
	GrpcChannelConfigurator userManagerConfigurator() {
		return GrpcChannelConfigurator.builder()
				.clientName(CLIENT_NAME_USER_MANAGER)
				.address(userManagerProperties.getAddress())
				.negotiationType(userManagerProperties.getNegotiationType())
				.build();
	}

	@Bean
	@ConditionalOnProperty("ozgcloud.file-manager.address")
	GrpcChannelConfigurator fileManagerConfigurator() {
		return GrpcChannelConfigurator.builder()
			.clientName(CLIENT_NAME_FILE_MANAGER)
			.address(fileManagerProperties.getAddress())
			.negotiationType(fileManagerProperties.getNegotiationType())
			.build();
	}

	@Bean
	GrpcChannelsProperties clientProperties(Collection<GrpcChannelConfigurator> configurators) {
		var properties = new GrpcChannelsProperties();
		configurators.forEach(configurator -> configurator.addToProperties(properties));

		return properties;
	}

	private void addVorgangManager(Map<String, GrpcChannelProperties> clientMap) {
		var channelProps = new GrpcChannelProperties();
		channelProps.setAddress(vorgangManagerProperties.getAddress());
		channelProps.setNegotiationType(vorgangManagerProperties.getNegotiationType());

		clientMap.put(CLIENT_NAME_VORGANG_MANAGER, channelProps);
	}

	private void addFileManager(Map<String, GrpcChannelProperties> clientMap) {
		var channelProps = new GrpcChannelProperties();
		channelProps.setAddress(fileManagerProperties.getAddress());
		channelProps.setNegotiationType(fileManagerProperties.getNegotiationType());

		clientMap.put(CLIENT_NAME_FILE_MANAGER, channelProps);
	}

	private void addCommandManager(Map<String, GrpcChannelProperties> clientMap) {
		var channelProps = new GrpcChannelProperties();
		channelProps.setAddress(commandManagerProperties.getAddress());
		channelProps.setNegotiationType(commandManagerProperties.getNegotiationType());

		clientMap.put(CLIENT_NAME_COMMAND_MANAGER, channelProps);
	}

	@Bean
	@ConditionalOnMissingBean
	OzgCloudCallContextProvider callContextProvider(ApplicationContext ctxt) {
		return new DefaultOzgCloudCallContextProvider(ctxt);
	}

	@Bean("ozgCloudVorgangService")
	@ConditionalOnProperty("ozgcloud.vorgang-manager.address")
	@ConditionalOnMissingBean(OzgCloudVorgangService.class)
	OzgCloudVorgangService grpcOzgCloudVorgangService(OzgCloudVorgangMapper mapper, OzgCloudVorgangStubMapper stubMapper,
		OzgCloudVorgangQueryMapper queryMapper, OzgCloudCallContextProvider contextProvider) {
		var channel = ManagedChannelBuilder.forTarget(vorgangManagerProperties.getAddress()).usePlaintext().build();
		var vorgangServiceStub = VorgangServiceGrpc.newBlockingStub(channel);
		return new GrpcOzgCloudVorgangService(vorgangServiceStub, mapper, stubMapper, queryMapper, contextProvider);
	}

	@Bean
	@ConditionalOnProperty("ozgcloud.vorgang-manager.address")
	@ConditionalOnMissingBean
	OzgCloudClientAttributeService grpcOzgCloudClientAttributeService(OzgCloudClientAttributeMapper clientAttributeMapper,
		OzgCloudCallContextProvider contextProvider) {
		var channel = ManagedChannelBuilder.forTarget(vorgangManagerProperties.getAddress()).usePlaintext().build();
		var blockingStub = ClientAttributeServiceGrpc.newBlockingStub(channel);
		return new OzgCloudClientAttributeGrpcService(blockingStub, clientAttributeMapper, contextProvider);
	}

	@Bean
	OzgCloudVorgangMapper ozgCloudVorgangMapper() {
		return Mappers.getMapper(OzgCloudVorgangMapper.class);
	}

	@Bean
	OzgCloudVorgangStubMapper ozgCloudVorgangStubMapper() {
		return Mappers.getMapper(OzgCloudVorgangStubMapper.class);
	}

	@Bean
	OzgCloudVorgangQueryMapper ozgCloudVorgangQueryMapper() {
		return Mappers.getMapper(OzgCloudVorgangQueryMapper.class);
	}

	@Bean
	OzgCloudClientAttributeMapper ozgCloudClientAttributeMapper() {
		return Mappers.getMapper(OzgCloudClientAttributeMapper.class);
	}

	@Bean("ozgCloudVorgangService")
	@ConditionalOnMissingBean(OzgCloudVorgangService.class)
	OzgCloudVorgangService dummyOzgCloudVorgangService() {
		return new DummyVorgangService();
	}

	@Bean
	OzgCloudVorgangIdMapper ozgCloudVorgangIdMapper() {
		return Mappers.getMapper(OzgCloudVorgangIdMapper.class);
	}

	@Bean
	UserProfileMapper ozgCloudUserProfileMapper() {
		return Mappers.getMapper(UserProfileMapper.class);
	}

	@Bean
	@ConditionalOnProperty("ozgcloud.command-manager.address")
	@ConditionalOnMissingBean(OzgCloudCommandService.class)
	OzgCloudCommandService grpcOzgCloudCommandService(CommandMapper commandMapper, OzgCloudCallContextProvider contextProvider) {
		var channel = ManagedChannelBuilder.forTarget(commandManagerProperties.getAddress()).usePlaintext().build();

		var commandServiceStub = CommandServiceGrpc.newBlockingStub(channel);

		return new GrpcOzgCloudCommandService(commandServiceStub, commandMapper, contextProvider,
				GrpcOzgCloudCommandService.DEFAULT_COMMAND_REQUEST_THRESHOLD_MILLIS);
	}

	@Bean
	@ConditionalOnProperty("ozgcloud.command-manager.address")
	CommandMapper commandMapper() {
		return Mappers.getMapper(CommandMapper.class);
	}

	@Bean
	@ConditionalOnProperty("ozgcloud.command-manager.address")
	AlfaService commandAlfaService(OzgCloudCommandService commandService) {
		return new CommandAlfaService(commandService);
	}

	@Bean
	@ConditionalOnMissingBean
	AlfaService alfaService() {
		return new DummyAlfaService();
	}

	@Bean
	@ConditionalOnProperty("ozgcloud.file-manager.address")
	OzgCloudFileMapper fileMapper() {
		return Mappers.getMapper(OzgCloudFileMapper.class);
	}

	@Bean
	@ConditionalOnMissingBean
	@ConditionalOnProperty("ozgcloud.user-manager.address")
	OzgCloudUserProfileService grpcOzgCloudUserProfileService(UserProfileMapper mapper, OzgCloudCallContextProvider contextProvider) {
		var channel = ManagedChannelBuilder.forTarget(userManagerProperties.getAddress()).usePlaintext().build();
		var userServiceStub = UserProfileServiceGrpc.newBlockingStub(channel);
		return new GrpcOzgCloudUserProfileService(userServiceStub, mapper, contextProvider);
	}

	@Bean
	@ConditionalOnProperty("ozgcloud.file-manager.address")
	@ConditionalOnMissingBean
	OzgCloudFileService grpcOzgCloudFileService(OzgCloudFileMapper mapper, OzgCloudCallContextProvider contextProvider) {
		var channel = ManagedChannelBuilder.forTarget(fileManagerProperties.getAddress()).usePlaintext().build();
		var blockingStub = BinaryFileServiceGrpc.newBlockingStub(channel);
		var asyncStub = BinaryFileServiceGrpc.newStub(channel);
		return new GrpcOzgCloudFileService(blockingStub, asyncStub, contextProvider, mapper);
	}
}
